/* 
Here is the ftp.c ... it's a little over 4K.
------8<-----------------------8<--------------------------8<-----------------
*/
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include <ctype.h>
#include <unistd.h>
#include "protocol.h"
#include "url.h"
#include "snarf.h"
#include "option.h"
#include "misc.h"

#define MAXLEN 4096

static int getresponse (int, char *) ;
off_t getsize(const char *);

int ftp_get (struct url u, char *outfile)
{
  struct hostent *host ;
  struct sockaddr_in sin ;
  int s, control, data, tmp ;
  char send_buf[MAXLEN] ; 
  char line[MAXLEN] ;
  unsigned char *a, *p ;
  int fd, open_flags = O_CREAT | O_WRONLY;
  int response;
  off_t file_offset;


  if(resume)
    open_flags |= O_APPEND;
  else
    open_flags |= O_TRUNC;

  /* get the host's IP addr */
  if((host = gethostbyname(u.u_host)) == NULL) 
    {
      fprintf(stderr, "%s: host not found.\n", u.u_host);
      return(0);
    }
  
  /* get the socket */
  if((control = socket(AF_INET, SOCK_STREAM, 0)) < 0) 
    {
      fprintf(stderr, "Unable to create socket.\n");
      return(0);
    }

  /* connect the socket, filling in the important stuff */
  sin.sin_family = AF_INET;
  sin.sin_port = htons(u.u_port);
  bcopy(host->h_addr, &sin.sin_addr, host->h_length);
  
  if(connect(control, (struct sockaddr *)&sin, sizeof(sin)) < 0)
    {
      fprintf(stderr, "Connection to host %s refused.\n", u.u_host);
      return(0);
    }

  /* Login as anonymous, change to reqd. directory and make type binary */

  if (getresponse (control, line) != 220) /* ready for new user */
    return 0 ;
  sprintf (send_buf, "USER anonymous\r\n") ;
  send (control, send_buf, strlen (send_buf), 0) ;

  response = getresponse(control, line);
  if (response != 331 && response != 230) /* login ok, enter passwd 
					     - or, you're logged in without 
					     further info necessary
					     - zpb */
    return 0 ;
  sprintf (send_buf, "PASS %s@\r\n", getpwuid(getuid())->pw_name) ;
  /* some ftp servers may require rest of address too - not done 
     - actually most are pretty flexible, and just warn you if you don't
     put in the whole thing; i've yet to see one that actually requires
     the whole thing
     - zpb */
  send (control, send_buf, strlen (send_buf), 0) ;
#ifdef DEBUG
  fprintf(stderr, "checking guest login ability\n");
#endif
  response = getresponse(control, line);
  if (response != 230 && response != 202) /* guest login ok 
					     - or no passwd needed
					     - zpb */
    return 0 ;
  sprintf (send_buf, "CWD %s\r\n", u.u_path) ;
  send (control, send_buf, strlen (send_buf), 0) ;

#ifdef DEBUG
  fprintf(stderr, "changing dirs\n");
#endif

  if (getresponse (control, line) != 250) /* cwd command successful */
    return 0 ;
  sprintf (send_buf, "TYPE I\r\n") ; /* always use binary */
  send (control, send_buf, strlen (send_buf), 0) ;
  if (getresponse (control, line) != 200) /* type set to I */
    return 0 ;


  /* now for the dirty part - create data connection and suck data from
     server */

  if((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) 
    {
      fprintf(stderr, "Unable to create socket, shitola.\n");
      return(0);
    }
  tmp = sizeof (sin) ; 
  /* We use PORT, rather than SO_REUSEADDR. However, you can easily modify
     the code if you want that */
  getsockname (control, (struct sockaddr *)&sin, &tmp) ; 
  sin.sin_port = 0 ; /* let system choose a port */
  if (bind (s, (struct sockaddr *)&sin, sizeof (sin)) < 0)
    {
      fprintf (stderr, "Can't bind.\n") ; 
      return 0 ;
    }
  getsockname (s, (struct sockaddr *)&sin, &tmp) ;
  listen (s, 1) ;
  a = (unsigned char *)&sin.sin_addr ;
  p = (unsigned char *)&sin.sin_port ;
  sprintf (send_buf, "PORT %d,%d,%d,%d,%d,%d\r\n", a[0], a[1], a[2], a[3],
	   p[0], p[1]) ;
  send (control, send_buf, strlen (send_buf), 0) ;
  if (getresponse (control, line) != 200) /* port command successful */
    return 0 ;
  
  /* get a file descriptor pointing at the right place */
  if(strcmp(outfile, "-") == 0)
    fd = 1;
  else
    if((fd = open(outfile, open_flags, 0644)) < 0) 
      {
	fprintf(stderr, "Unable to open output file %s for writing.\n", outfile);
	return(0);
      }

  /* if the resume option is set, get the size of the local file and tell
     the ftp server to restart the transfer at that offset in the remote
     file
     - xach */
  if(resume){
    file_offset = getsize(u.u_file);
    sprintf(send_buf, "REST %ld\r\n", file_offset);
    send(control, send_buf, strlen(send_buf), 0);
    if(getresponse(control, line) != 350)
      return 0 ;
  }

  /* if URL ends in a /, we ask for a directory listing */    
  if (strcmp (u.u_file, ""))
    sprintf (send_buf, "RETR %s\r\n", u.u_file) ;
  else
    sprintf (send_buf, "LIST\r\n") ;
  send (control, send_buf, strlen (send_buf), 0) ;
  if (getresponse (control, line) != 150) /* opening connection */
    return 0 ;

  data = accept (s, (struct sockaddr *)&sin, &tmp) ;
  if (data < 0)
    {
      fprintf (stderr, "Accept error.\n") ;
      return 0 ;
    }

  dump_data(data, fd); /* suck data from server and dump it to fd */

/*  
    while((tmp = read(data, line, sizeof(line))) > 0){
    write(fd, line, tmp); 
    if(hash)
      fprintf(stderr, "#");
  }
*/
  
  getresponse (control, line) ;
  sprintf (send_buf, "QUIT\r\n") ;
  send (control, send_buf, strlen (send_buf), 0) ;
  getresponse (control, line) ;

  close (fd) ;
  close (control) ;
  close (data) ;
  close (s) ;
  return 1 ;
}


/* Gets replies from server, ignores informational lines, and returns
   reply code */

static int getresponse (int fd, char *line) 
{
  int read ;

  while ((read = readline (fd, line)) > 0)
    {
#ifdef DEBUG
      fprintf(stderr, "%c\n", line[0]);
#endif
      if (verbose) 
	fprintf (stderr, "%s", line) ;
      if (line[3] != '-' && (isdigit(line[0])))
	                                    /* if it's not some informative 
					       message */
	/* - actually, microsoft ftp servers (and perhaps others) are lame
	   enough to reply with status messages that have a space as the 
	   first letter. stupid! i think. -zpb 
	   whoops...the rfc says you can do that, heh. you have an initial
	   response that starts with a number and a dash, then iwhatever you
	   like, and then a line with the actual status number - zpb */
	break ;
    }
  if (read <= 0) 
    return read ;
  else
    return atoi (line) ;
}


/* this function returns the size of a file (if it exists) or
   zero if it doesnt */
off_t getsize(const char *file){
  struct stat file_info;

  if(stat(file, &file_info) == -1)
    return(0);
  else
    return(file_info.st_size);
}
/*
------------>8--------------------->8------------------------>8------------

							-- Ganesh
						 (ganesh@cse.iitb.ernet.in)
----------------------------------------------------------------------------
"Today for show and tell, I've brought a tiny marvel of nature : a single
snowflake. I think we might all learn a lesson from how this utterly unique
and exquisite crystal turns into an ordinary, boring molecule of water, just
like every other one, when you bring it in the classroom. And now, while the
analogy sinks in, I'll be leaving you drips and going outside."
			-- Calvin and Hobbes, "Homicidal Psycho Jungle Cat"
----------------------------------------------------------------------------
*/
